@@ -23,8 +23,6 @@ struct foo;
 #define SZABSPATH(x) (CACHE_BASE+(x)).c_str()
 #define SABSPATH(x) (CACHE_BASE+(x))
 
-#define REDIRMAX acfg::redirmax
-
 #define FAKEDATEMARK "Sat, 26 Apr 1986 01:23:39 GMT+3"
 
 // caching all relevant file identity data and helper flags in such entries
@@ -115,7 +113,7 @@ protected:
 		EIDX_ARCHLXDB,
 		EIDX_CYGSETUP,
 		EIDX_SUSEREPO,
-		EIDX_SUSEOTHER,
+		EIDX_XMLRPMLIST,
 		EIDX_RFC822WITHLISTS,
 		EIDX_TRANSIDX
 	};
