@@ -16,19 +16,6 @@
 
 #endif
 
-
-#if defined(__CYGWIN32__) || defined(__CYGWIN__)
-#include <cygwin/in.h>
-#endif
-
-#ifndef AI_NUMERICSERV
-#define AI_NUMERICSERV 0
-#endif
-
-#ifndef AI_ADDRCONFIG
-#define AI_ADDRCONFIG 0
-#endif
-
 class CAddrInfo;
 
 class CAddrInfo : public lockable
