@@ -71,7 +71,7 @@ struct tFingerprint {
 	}
 	mstring GetCsAsString() const
 	{
-		return CsBinToString(csum, CSTYPE_MD5==csType?16:20);
+		return BytesToHexString(csum, CSTYPE_MD5==csType?16:20);
 	}
 	operator mstring() const
 	{
