@@ -65,6 +65,18 @@ class dlcon : public lockable
 
     	UINT ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueue &qActive);
 
+    	// Disable pipelining for the next # requests. Actually used as crude workaround for the
+    	// concept limitation (because of automata over a couple of function) and its
+    	// impact on download performance.
+    	// The use case: stupid web servers that redirect all requests do that step-by-step, i.e.
+    	// they get a bunch of requests but return only the first response and then flush the buffer
+    	// so we process this response and wish to switch to the new target location (dropping
+    	// the current connection because we don't keep it somehow to background, this is the only
+    	// download agent we have). This manner perverts the whole principle and causes permanent
+    	// disconnects/reconnects. In this case, it's beneficial to disable pipelining and send
+    	// our requests one-by-one. This is done for a while (i.e. the valueof(m_nDisablePling)/2 )
+    	// times before the operation mode returns to normal.
+    	int m_nDisablePling;
 };
 
 #endif
