@@ -13,7 +13,8 @@ public:
 		list,
 		purge,
 		listDamaged,
-		purgeDamaged
+		purgeDamaged,
+		truncDamaged
 	};
 	expiration(int fd, workType type);
 	virtual ~expiration();
