@@ -68,7 +68,6 @@ public:
 
 	uint64_t m_nIncommingCount;
 	off_t m_nSizeSeen;
-	mstring m_sPathAbs, m_sPathRel;
 	
 	bool m_bCheckFreshness;
 
@@ -91,6 +90,10 @@ protected:
 	time_t m_nTimeExpireAt; //! when to consider this item obsolete
 	static time_t m_nEarliestExpiration; //! when to consider a closer look for expired items
 
+	mstring m_sPathAbs, m_sPathRel;
+
+	time_t m_nTimeDlDone;
+
 public:
 	virtual void Unreg() {}; // to decrease dependent user count when used in global contents
 
@@ -108,6 +111,7 @@ class fileitem_with_storage : public fileitem_base
 {
 public:
 	inline fileitem_with_storage(cmstring &s) : fileitem_base(s) {};
+	virtual ~fileitem_with_storage();
 	// send helper like wrapper for sendfile. Just declare virtual here to make it better customizable later.
 	virtual ssize_t SendData(int confd, int filefd, off_t &nSendPos, size_t nMax2SendNow);
 	virtual SuccessWithTransErrorFlag DownloadStartedStoreHeader(const header & h, const char *pNextData,
@@ -115,7 +119,8 @@ public:
 	virtual bool StoreFileData(const char *data, unsigned int size);
 };
 
-class fileitem;
+typedef MYSTD::pair<int,tFileItemPtr> tFileRefEntry;
+typedef MYSTD::multimap<mstring, tFileRefEntry> tFiGlobMap;
 
 //! This is a fileitem class which is aware of other items and download sources,
 //! includes global download state management
@@ -124,27 +129,24 @@ class fileitem : public fileitem_with_storage
 public:
 	virtual void Unreg(); // decrease dependent user count
 
-	//! Call Unreg directly, or return immediately but call it few seconds later
-	static void UnregOneASAP(tFileItemPtr);
-
 	// related to GetRegisteredFileItem but used for registration of custom file item
 	// implementations created elsewhere (which still need to obey regular work flow)
 	static bool RegisterFileItem(SHARED_PTR<fileitem> spCustomFileItem);
 
-	// public constructor wrapper, get a unique object from the map
-	static tFileItemPtr GetRegisteredFileItem(mstring sPath); // also opens the cached file and increases reference count
+	// public constructor wrapper, get a unique object from the map or a new one
+	static tFileItemPtr GetRegisteredFileItem(cmstring &sPath, bool bConsiderAltStore);
 
 	//! @return: true iff there is still something in the pool for later cleaning
 	static time_t DoDelayedUnregAndCheck();
 
-protected:
-	inline fileitem(cmstring s) : fileitem_with_storage(s) {};
+	static void dump_status();
 
+protected:
+	fileitem(cmstring &s);
+	tFiGlobMap::iterator m_globRef;
 };
 #endif
 
-extern lockable mapLck;
-
 #endif
 
 
