@@ -78,16 +78,13 @@ class job {
 
       const char * BuildAndEnqueHeader(const FiStatus &fistate, const off_t &nGooddataSize, header& respHead);
       FiStatus _SwitchToPtItem(const mstring &fileLoc);
-      void SetErrorResponse(const char * errorLine);
+      void SetErrorResponse(const char * errorLine, const char *szLocation=NULL);
       void HandleLocalDownload(const mstring &visPath,
     			const mstring &fsBase, const mstring &fsSubpath);
 
       bool ParseRange();
 
       off_t m_nReqRangeFrom, m_nReqRangeTo;
-
-      int m_nRedirRemaining;
-      bool CreateRedirect(header &respHead);
 };
 
 #endif
