@@ -27,8 +27,13 @@ protected:
 	virtual void UpdateFingerprint(const mstring &sPathRel, off_t nOverrideSize,
 				uint8_t *pOverrideSha1, uint8_t *pOverrideMd5);
 
-	bool m_bCalcSize, m_bSkipIxUpdate, m_bDoDownload, m_bAsNeeded;
-	off_t m_totalneed;
-	tStrSet m_cachedDirs;
+	bool m_bCalcSize, m_bSkipIxUpdate, m_bDoDownload, m_bAsNeeded, m_bUseDelta;
+	off_t m_totalSize, m_totalHave;
+	tStrSet m_pathFilter;
+
+	const tHttpUrl *m_pDeltaSrc;
+	tStrPos m_repCutLen;
+
+	bool ConfigDelta(cmstring &sPathRel);
 };
 #endif /* MIRROR_H_ */
