@@ -11,6 +11,7 @@
 #include "sockio.h"
 
 class tcpconnect;
+class fileitem_base;
 typedef SHARED_PTR<tcpconnect> tTcpHandlePtr;
 
 class tcpconnect
@@ -45,11 +46,11 @@ protected:
 
 	acfg::tRepoData::IHookHandler *m_pConnStateObserver;
 
-	mstring m_lastFileKeyPath;
+	WEAK_PTR<fileitem_base> m_lastFile;
 
 public:
 	//! @brief Remember the file name belonging to the recently initiated transfer
-	inline void NameLastFile(cmstring& keypath) { m_lastFileKeyPath=keypath; }
+	inline void KnowLastFile(WEAK_PTR<fileitem_base> spRemItem) { m_lastFile = spRemItem; }
 	//! @brief Invalidate (truncate) recently touched file
 	void KillLastFile();
 };
