@@ -112,6 +112,7 @@ MapNameToInt n2iTbl[] = {
 		,{ "FreshIndexMaxAge", &maxtempdelay, NULL, 10}
 		,{ "RedirMax", &redirmax, NULL, 10}
 		,{ "VfileUseRangeOps", &vrangeops, NULL, 10}
+		,{ "ResponseFreezeDetectTime", &stucksecs, NULL, 10}
 
 		,{ "DirPerms", &dirperms, NULL, 8}
 		,{ "FilePerms", &fileperms, NULL, 8}
@@ -1150,7 +1151,7 @@ void PostProcConfig()
    }
 
    if(redirmax == RESERVED_DEFVAL)
-	   redirmax = forcemanaged ? 0 : 5;
+	   redirmax = forcemanaged ? 0 : ACFG_REDIRMAX_DEFAULT;
 
    if (acfg::debug >= LOG_MORE)
 	{
@@ -1221,7 +1222,7 @@ void PostProcConfig()
 
 time_t ExecutePostponed()
 {
-	time_t ret(g_victor.never()), now(time(0));
+	time_t ret(END_OF_TIME), now(time(0));
 	for (tRepMap::iterator it = repoparms.begin(); it != repoparms.end(); ++it)
 	{
 		if (!it->second.m_pHooks)
