@@ -21,7 +21,7 @@ confdir, adminauth, bindaddr, mirrorsrcs, suppdir("/usr/lib/apt-cacher-ng");
 //#define COMPONENT_OPTIONAL "(-[a-z0-9-])"
 //#define PARANOIASOURCE "(\\.orig|\\.debian)"
 
-string pfilepat(".*(\\.d?deb|\\.rpm|\\.dsc|\\.tar"COMPRLIST"(\\.gpg)?"
+string pfilepat(".*(\\.d?deb|\\.rpm|\\.drpm|\\.dsc|\\.tar"COMPRLIST"(\\.gpg)?"
 		"|\\.diff"COMPRLIST"|\\.jigdo|\\.template|changelog|copyright"
 		"|\\.udeb|\\.debdelta|\\.diff/.*\\.gz|(Devel)?ReleaseAnnouncement(\\?.*)?"
 		"|[a-f0-9]+-(susedata|updateinfo|primary|deltainfo).xml.gz" //opensuse, index data, hash in filename
@@ -64,7 +64,7 @@ keepnver(0), maxtempdelay(27), vrangeops(1);
 
 int dnscachetime(1800), dlbufsize(70000), exfailabort(1), exporigin(false), numcores(1),
 logxff(false), oldupdate(false), recompbz2(false), nettimeout(60), updinterval(0),
-forwardsoap(RESERVED_DEFVAL), usewrap(RESERVED_DEFVAL), redirmax(RESERVED_DEFVAL);
+forwardsoap(RESERVED_DEFVAL), usewrap(RESERVED_DEFVAL), redirmax(RESERVED_DEFVAL), stucksecs(500);
 
 string agentname("Debian Apt-Cacher-NG/" ACVERSION);
 string remoteport("80"), port("3142");
