@@ -91,15 +91,12 @@ void err(const char *msg, const char *client)
 	fErr << msg << '\n';
 
 #ifdef DEBUG
-	if(acfg::debug>=LOG_DEBUG)
+	if(acfg::debug & LOG_DEBUG)
 		cerr << buf << msg <<endl;
 #endif
 
-#ifdef DEBUG // forced flush is optional in debug build...
-	if(acfg::debug & LOG_FLUSH)
+	if(acfg::debug & LOG_DEBUG)
 		fErr.flush();
-#endif
-
 }
 
 void flush()
