@@ -25,6 +25,7 @@ static void usage();
 static void _TestCacheDir();
 void term_handler(int signum);
 void log_handler(int signum);
+void dump_handler(int signum);
 void check_algos();
 
 void DispatchAndRunMaintTask(cmstring &cmd, int fd, const char *auth);
@@ -121,6 +122,8 @@ int main(int argc, char **argv)
 	sigaction(SIGTERM, &act, NULL);
 	sigaction(SIGINT, &act, NULL);
 	sigaction(SIGQUIT, &act, NULL);
+
+	act.sa_handler = &dump_handler;
 	sigaction(SIGUSR2, &act, NULL);
 
 	act.sa_handler = &log_handler;
@@ -185,8 +188,10 @@ int main(int argc, char **argv)
 	acfg::PostProcConfig();
     
     _TestCacheDir();
-    //sleep(20);
-    //g_victor.Start();
+
+	extern mstring sReplDir;
+	DelTree(acfg::cacheDirSlash+sReplDir);
+
 	conserver::Setup();
 	
 	if (bForceCleanup)
@@ -253,6 +258,7 @@ static void _TestCacheDir()
 	}
 	cerr << "Cache directory not writable. Check the permissions of "
 		<< acfg::cachedir << "!" <<endl;
+
 	exit(1);
 }
 
