@@ -102,7 +102,7 @@ void tWuiBgTask::Run(const string &cmd)
 			m_pTracker.reset(new tProgressTracker);
 			msg.clear();
 			time_t nMaintId=time(0);
-			msg<<acfg::logdir<<CPATHSEP<<"maint_" << nMaintId << ".log";
+			msg<<acfg::logdir<<CPATHSEP<<"maint_" << nMaintId << ".log.html";
 			m_reportStream.open(msg.c_str(), ios::out);
 			if(m_reportStream.is_open())
 			{
@@ -141,7 +141,7 @@ void tWuiBgTask::Run(const string &cmd)
 
 			if(!fd)
 			{
-				msg<<acfg::logdir<<CPATHSEP<<"maint_" << pTracked->id << ".log";
+				msg<<acfg::logdir<<CPATHSEP<<"maint_" << pTracked->id << ".log.html";
 				fd=open(msg.c_str(), O_RDONLY);
 				msg.clear();
 				if(fd>=0)
@@ -243,7 +243,7 @@ void tWuiBgTask::DumpLog(time_t id)
 		return;
 
 	tSS path(acfg::logdir.length()+24);
-	path<<acfg::logdir<<CPATHSEP<<"maint_" << id << ".log";
+	path<<acfg::logdir<<CPATHSEP<<"maint_" << id << ".log.html";
 	if (!reader.OpenFile(path))
 		tWUIPage::SendChunk(_SZ2PS("Log not available"));
 	else
