@@ -262,136 +262,108 @@ bool tCacheMan::Download(const MYSTD::string & sFilePathRel, bool bIndexFile,
 #define GOTOREPMSG(x) {sErr = x; bSuccess=false; if(fi) {fi->Unreg(); fi.reset();} goto rep_dlresult; }
 
 	tHttpUrl origURL;
-	for(int i=0; i<=REDIRMAX && !bSuccess; ++i)
+	tFileItemPtr fi;
+	// handle alternative behavior in the first or last cycles
+	if (pForcedURL && !origURL.SetHttpUrl(pForcedURL))
+		return false;
+
+	fi = pItemDelg ? *pItemDelg : fileitem::GetRegisteredFileItem(sFilePathRel, false);
+
+	if (!fi)
 	{
-		tFileItemPtr fi;
-		// handle alternative behavior in the first or last cycles
-		if(0==i && pForcedURL)
+		if (nVerbosity)
+			SendFmt() << "Checking " << sFilePathRel << "...\n"; // just display the name ASAP
+		GOTOREPMSG(" could not create file item handler.");
+	}
+	if (bIndexFile && m_bForceDownload)
+	{
+		if (!fi->SetupClean())
+			GOTOREPMSG("Item busy, cannot reload");
+		if (nVerbosity)
+			SendFmt() << "Downloading " << sFilePathRel << "...\n";
+	}
+	else
+	{
+		FiStatus initState = fi->Setup(bIndexFile);
+		if (FIST_COMPLETE == initState)
 		{
-			if(!origURL.SetHttpUrl(pForcedURL))
-				return false;
-			continue;
+			SendFmt() << "Checking " << sFilePathRel << "... (complete)<br>\n";
+			return true;
 		}
-		if(i==REDIRMAX)
-			GOTOREPMSG("too many redirections.");
-
-		fi = pItemDelg ? *pItemDelg : fileitem::GetRegisteredFileItem(sFilePathRel);
-
-		if(!fi)
+		if (initState > FIST_COMPLETE)
 		{
-			if(nVerbosity)
-				SendFmt()<<"Checking "<<sFilePathRel<<"...\n"; // just display the name ASAP
-			GOTOREPMSG(" could not create file item handler.");
+			GOTOREPMSG(fi->GetHeader().frontLine);
 		}
-		if(bIndexFile && m_bForceDownload)
+		if (nVerbosity)
+			SendFmt() << (bIndexFile ? "Checking/Updating " : "Downloading ") << sFilePathRel
+					<< "...\n";
+	}
+
+	if (!FindURL_InitDload(fi, sFilePathRel, sErr, &origURL))
+		GOTOREPMSG("Failed to resolve source URL");
+
+	if (origURL.sHost.empty())
+		GOTOREPMSG("Failed to resolve original URL")
+
+	m_pDlcon->AddJob(fi, origURL);
+
+	m_pDlcon->WorkLoop();
+	if (fi->WaitForFinish(NULL) == FIST_COMPLETE)
+	{
+		bSuccess = true;
+		if (nVerbosity)
+			SendFmt() << "<i>(" << fi->GetTransferCount() / 1024 << "KiB)</i>\n";
+		fi->Unreg(); // anyhow, release our pointer
+		fi.reset();
+	}
+	else
+	{
+		if (IsDeprecatedArchFile(sFilePathRel))
 		{
-			if(!fi->SetupClean())
-				GOTOREPMSG("Item busy, cannot reload");
-			if(nVerbosity)
-				SendFmt()<<"Downloading "<<sFilePathRel<<"...\n";
+			if (nVerbosity == VERB_SHOW)
+				SendFmt() << "<i>(no longer available)</i>\n";
+			m_forcedTrashMap[sFilePathRel] = true;
+			bSuccess = true;
+		}
+		else if (flags.forgiveDlErrors)
+		{
+			bSuccess = true;
+			if (nVerbosity == VERB_SHOW)
+				SendFmt() << "<i>(ignored)</i>\n";
 		}
 		else
-		{
-			FiStatus initState = fi->Setup(bIndexFile);
-			if(FIST_COMPLETE == initState)
-			{
-				SendFmt()<<"Checking "<<sFilePathRel<<"... (complete)<br>\n";
-				return true;
-			}
-			if(initState > FIST_COMPLETE)
-			{
-				GOTOREPMSG(fi->GetHeader().frontLine);
-			}
-			if(nVerbosity)
-				SendFmt()<< (bIndexFile?"Checking/Updating " : "Downloading ")
-				<<sFilePathRel<<"...\n";
-		}
+			GOTOREPMSG(fi->GetHttpMsg());
 
-		if(i == 0)
-		{
-			if( ! FindURL_InitDload(fi, sFilePathRel, sErr, &origURL))
-				GOTOREPMSG("Failed to resolve source URL");
-		}
-		else if(origURL.sHost.empty())
-			GOTOREPMSG("Failed to resolve original URL")
-		else
-			m_pDlcon->AddJob(fi, origURL);
-
-		m_pDlcon->WorkLoop();
-		int httpCode;
-		if(fi->WaitForFinish(&httpCode) != FIST_COMPLETE)
-			httpCode=555;
-
-		switch(httpCode)
-		{
-		case 200:
-			bSuccess=true;
-			if(nVerbosity)
-				SendFmt() << "<i>(" << fi->GetTransferCount() / 1024 << "KiB)</i>\n";
-			fi->Unreg(); // anyhow, release our pointer
-			fi.reset();
-			break;
-		case 301:
-		case 302:
-		case 307:
-		{
-			header h=fi->GetHeader();
-			fi->Unreg(); // anyhow, release our pointer
-			fi.reset();
-			if(h.h[header::LOCATION] && origURL.SetHttpUrl(h.h[header::LOCATION]))
-				continue;
-		}
-		default:
-		{
-			if (IsDeprecatedArchFile(sFilePathRel))
-			{
-				if(nVerbosity == VERB_SHOW)
-					SendFmt() << "<i>(no longer available)</i>\n";
-				m_forcedTrashMap[sFilePathRel] = true;
-				bSuccess = true;
-			}
-			else if (flags.forgiveDlErrors)
-			{
-				bSuccess = true;
-				if(nVerbosity == VERB_SHOW)
-					SendFmt() << "<i>(ignored)</i>\n";
-			}
-			else
-				GOTOREPMSG(fi->GetHttpMsg());
-
-			fi->Unreg(); // anyhow, release our pointer
-			fi.reset();
-			break;
-		}
-		}
+		fi->Unreg(); // anyhow, release our pointer
+		fi.reset();
 	}
 
 	rep_dlresult:
 
 	if (bSuccess && bIndexFile)
-		SetFlags(sFilePathRel).uptodate=true;
+		SetFlags(sFilePathRel).uptodate = true;
 
 	UpdateFingerprint(sFilePathRel, -1, NULL, NULL);
 
-	if(m_bVerbose || !bSuccess)
+	if (m_bVerbose || !bSuccess)
 	{
-		m_bShowControls=true;
+		m_bShowControls = true;
 
-		if(!bSuccess)
+		if (!bSuccess)
 		{
-			if(nVerbosity == VERB_SHOW)
+			if (nVerbosity == VERB_SHOW)
 			{
-				if(sErr.empty())
+				if (sErr.empty())
 					sErr = "Download error";
 				SendFmt() << "<font color=\"red\"><b>" << sErr << "</b></font>\n";
 			}
 		}
 
-		if(nVerbosity == VERB_SHOW)
+		if (nVerbosity == VERB_SHOW)
 			AddDelCbox(sFilePathRel);
 	}
 
-	if(nVerbosity!=VERB_QUIET)
+	if (nVerbosity != VERB_QUIET)
 		SendChunk("<br>\n");
 
 	return bSuccess;
@@ -676,6 +648,7 @@ class fakeCon : public ::tCacheMan::tGetItemDelegate
 		};
 		return tFileItemPtr(static_cast<fileitem_base*>(new tHeadOnlyStorage(*this)));
 	}
+	virtual ~fakeCon() {}
 };
 
 bool tCacheMan::GetAndCheckHead(cmstring & sTempDataRel, cmstring &sReferencePathRel,
@@ -816,7 +789,7 @@ bool tCacheMan::Propagate(cmstring &donorRel, tContId2eqClass::iterator eqClassI
 	struct keeper
 	{
 		tFileItemPtr pItem;
-		keeper(const string &path, bool y) { if(y) pItem=fileitem::GetRegisteredFileItem(path); }
+		keeper(const string &path, bool y) { if(y) pItem=fileitem::GetRegisteredFileItem(path, false); }
 		~keeper() { if(pItem) pItem->Unreg(); }
 	} src(donorRel, GetFlags(donorRel).uptodate);
 
@@ -1436,7 +1409,7 @@ tCacheMan::enumIndexType tCacheMan::GuessIndexTypeFromURL(const mstring &sPath)
 
 	if (sPureIfileName.length() > 50 && endsWithSzAr(sPureIfileName, ".xml") && sPureIfileName[40]
 			== '-')
-		return EIDX_SUSEOTHER;
+		return EIDX_XMLRPMLIST;
 
 	if (sPureIfileName == "Sources")
 		return EIDX_SOURCES;
@@ -1667,8 +1640,8 @@ bool tCacheMan::ParseAndProcessIndexFile(ifileprocessor &ret, const MYSTD::strin
 			}
 		}
 		break;
-	case EIDX_SUSEOTHER:
-		LOG("SUSE list file, pickup any valid filename ending in .rpm");
+	case EIDX_XMLRPMLIST:
+		LOG("XML based file list, pickup any valid filename ending in .rpm");
 		while(reader.GetOneLine(sLine))
 		{
 			if(CheckAbortCondition())
@@ -1678,13 +1651,16 @@ bool tCacheMan::ParseAndProcessIndexFile(ifileprocessor &ret, const MYSTD::strin
 			{
 				cmstring tok(split);
 				LOG("testing filename: " << tok);
-				if(!endsWithSzAr(tok, ".rpm"))
-					continue;
-				LOG("RPM basename: " << tok);
-				info.sFileName = tok;
-				info.sDirectory = sCurFilesReferenceDirRel;
-				ret.HandlePkgEntry(info, false);
-				info.SetInvalid();
+				if (endsWithSzAr(tok, ".rpm")
+						|| endsWithSzAr(tok, ".drpm")
+						|| endsWithSzAr(tok, ".srpm"))
+				{
+					LOG("RPM basename: " << tok);
+					info.sFileName = tok;
+					info.sDirectory = sCurFilesReferenceDirRel;
+					ret.HandlePkgEntry(info, false);
+					info.SetInvalid();
+				}
 			}
 		}
 		break;
