@@ -117,7 +117,7 @@ CAddrInfo::SPtr CAddrInfo::CachedResolve(const string & sHostname, const string
 time_t CAddrInfo::ExpireCache()
 {
 	lockguard g(&lockDnsCache);
-	time_t now(time(0)), ret(g_victor.never());
+	time_t now(time(0)), ret(END_OF_TIME);
 
 	for(map<string, CAddrInfo::SPtr>::iterator it=mapDnsCache.begin();
 			it!=mapDnsCache.end(); )
