@@ -23,7 +23,7 @@ cleaner g_victor;
 
 cleaner::cleaner() : m_thr(0), m_bTerminating(false)
 {
-	m_nextRun = never();
+	m_nextRun = END_OF_TIME;
 }
 
 void cleaner::WorkLoop()
@@ -47,12 +47,12 @@ void cleaner::WorkLoop()
 					return;
 				continue;
 			}
-			m_nextRun = never();
+			m_nextRun = END_OF_TIME;
 			// can do the work now...
 		}
 
 		time_t nextTimeProposal = fileitem::DoDelayedUnregAndCheck();
-		time_t nextTimeCand = min(nextTimeProposal, never());
+		time_t nextTimeCand = min(nextTimeProposal, END_OF_TIME);
 		nextTimeProposal = acfg::ExecutePostponed();
 		nextTimeCand = min(nextTimeProposal, nextTimeCand);
 		nextTimeProposal = CAddrInfo::ExpireCache();
