@@ -217,8 +217,8 @@ void con::WorkLoop() {
         ldbg("select con");
 
         struct timeval tv;
-        tv.tv_sec = 90;
-        tv.tv_usec = 0;
+        tv.tv_sec = acfg::nettimeout;
+        tv.tv_usec = 23;
         int ready = select(maxfd+1, &rfds, &wfds, NULL, &tv);
         
         if(ready == 0)
@@ -326,7 +326,17 @@ void con::WorkLoop() {
 
 				ldbg("Parsed REQUEST:" << m_pTmpHead->frontLine);
 				ldbg("Rest: " << inBuf.size());
-				AddNewJob(m_pTmpHead);
+
+				{
+					job * j = new job(m_pTmpHead, this);
+
+					j->PrepareDownload();
+					m_jobs2send.push_back(j);
+#ifdef DEBUG
+					m_nProcessedJobs++;
+#endif
+				}
+
 				m_pTmpHead=NULL; // owned by job now
 			}
         	MYCATCH(bad_alloc&)
@@ -453,16 +463,3 @@ void con::__tlogstuff::reset(const MYSTD::string &pNewFile, const MYSTD::string
 	sumIn=0;
 	sumOut=0;
 }
-
-job* con::AddNewJob(header *pHead)
-				{
-					job * j = new job(pHead, this);
-
-					j->PrepareDownload();
-					m_jobs2send.push_back(j);
-#ifdef DEBUG
-					m_nProcessedJobs++;
-#endif
-					return j;
-
-}
