@@ -17,7 +17,7 @@ using namespace MYSTD;
 static const cmstring sGenericError("567 Unknown download error occured");
 
 dlcon::dlcon(bool bManualExecution, string *xff) :
-		m_bStopASAP(false), m_bManualMode(bManualExecution)
+		m_bStopASAP(false), m_bManualMode(bManualExecution), m_nDisablePling(0)
 {
 	LOGSTART("dlcon::dlcon");
 	m_wakepipe[0] = m_wakepipe[1] = -1;
@@ -48,6 +48,7 @@ struct tDlJob
 #define HINT_SWITCH 32
 #define EFLAG_LOST_CON 64
 #define HINT_KILL_LAST_FILE 128
+#define HINT_TGTCHANGE 256
 
 	inline const mstring GetPeerName()
 	{
@@ -83,7 +84,9 @@ struct tDlJob
 
 	tDlState m_DlState;
 
-	void Init()
+	int m_nRedirRemaining;
+
+	inline void Init()
 	{
 		m_nRest = 0;
 		m_eReconnectASAP = 0;
@@ -93,16 +96,18 @@ struct tDlJob
 			m_pStorage->IncDlRefCount();
 	}
 
-	tDlJob(dlcon *p, tFileItemPtr pFi, const tHttpUrl& uri) :
+	inline tDlJob(dlcon *p, tFileItemPtr pFi, const tHttpUrl& uri, int redirmax) :
 			m_pStorage(pFi), m_pBEdata(NULL),
-			m_fileUri(uri)
+			m_fileUri(uri),
+			m_nRedirRemaining(redirmax)
 	{
 		Init();
 	}
 
-	tDlJob(dlcon *p, tFileItemPtr pFi, const acfg::tRepoData * pBackends,
-			const MYSTD::string & sPath) :
-			m_pStorage(pFi), m_pBEdata(pBackends), m_fileUri("", sPath)
+	inline tDlJob(dlcon *p, tFileItemPtr pFi, const acfg::tRepoData * pBackends,
+			const MYSTD::string & sPath, int redirmax) :
+			m_pStorage(pFi), m_pBEdata(pBackends), m_fileUri("", sPath),
+			m_nRedirRemaining(redirmax)
 	{
 		Init();
 	}
@@ -112,6 +117,46 @@ struct tDlJob
 		if (m_pStorage && FIST_COMPLETE != m_pStorage->GetStatus())
 			m_pStorage->DecDlRefCount(sErrorMsg.empty() ? sGenericError : sErrorMsg);
 	}
+
+	inline bool RewriteSource(const char *pNewUrl)
+	{
+
+		if (--m_nRedirRemaining <= 0)
+		{
+			sErrorMsg = "500 Bad redirection (loop)";
+			return false;
+		}
+
+		if (!pNewUrl)
+		{
+			sErrorMsg = "500 Bad redirection (empty)";
+			return false;
+		}
+
+		// it's previous server's job to not send any crap. Just make sure that
+		// the format is right for further processing here.
+		mstring s;
+		for (const char *p = pNewUrl; *p; ++p)
+		{
+			if (isspace((unsigned char) *p))
+			{
+				s += "%";
+				s += BytesToHexString((uint8_t*) p, 1);
+			}
+			else
+				s += *p;
+		}
+
+		m_pBEdata = NULL;
+		m_pCurBackend = NULL;
+		if (!m_fileUri.SetHttpUrl(s))
+		{
+			sErrorMsg = "500 Bad redirection (invalid URL)";
+			return false;
+		}
+
+		return true;
+	}
 //
 //acfg::tRepoData::IHookHandler * tDlJob::GetConnStateObserver()
 //{
@@ -127,10 +172,9 @@ struct tDlJob
 //}
 
 // needs connectedHost, blacklist, output buffer from the parent, proxy mode?
-	void AppendRequest(tSS &head, cmstring &xff)
+	inline void AppendRequest(tSS &head, cmstring &xff)
 	{
 		LOGSTART("tDlJob::AppendRequest");
-
 		bool proxied = ! acfg::proxy_info.sHost.empty();
 
 		head << (m_pStorage->m_bHeadOnly ? "HEAD " : "GET ");
@@ -270,7 +314,7 @@ struct tDlJob
 	 *
 	 * Process new incoming data and write it down to disk or other receivers.
 	 */
-	uint_fast8_t ProcessIncomming(acbuf & inBuf)
+	UINT ProcessIncomming(acbuf & inBuf, bool bOnlyRedirections)
 	{
 		LOGSTART("tDlJob::ProcessIncomming");
 		if (!m_pStorage)
@@ -314,15 +358,32 @@ struct tDlJob
 				}
 				ldbg("GOT, parsed: " << h.frontLine);
 
+				int st = h.getStatus();
+
+				// special handling for redirection, see caller ExchangeData() further processing
+				if(st == 301 || st == 302 || st == 307)
+				{
+					if(!acfg::redirmax)
+					{
+						ldbg("Internal redirection disabled by administrator, forward it to the user");
+					}
+					else if(RewriteSource(h.h[header::LOCATION]))
+						return HINT_TGTCHANGE;
+					else
+						return EFLAG_JOB_BROKEN;
+				}
+				else if(bOnlyRedirections)
+					return EFLAG_LOST_CON; // any error, just not tgtchange or broken
+
 				// explicitly blacklist mirror if key file is missing
-				if (h.getStatus() >= 400 && m_pBEdata)
+				if (st >= 400 && m_pBEdata)
 				{
 					for (tStrVecIterConst it = m_pBEdata->m_keyfiles.begin();
 							it != m_pBEdata->m_keyfiles.end(); ++it)
 					{
 						if (endsWith(m_fileUri.sPath, *it))
 						{
-							sErrorMsg = "500 Bad mirror";
+							sErrorMsg = "500 Keyfile missing, mirror blacklisted";
 							return HINT_DISCON | EFLAG_MIRROR_BROKEN;
 						}
 					}
@@ -342,7 +403,7 @@ struct tDlJob
 					m_DlState = STATE_FINISHJOB;
 				}
 				// the only case where we expect a 304
-				else if(h.getStatus() == 304 && acfg::vrangeops == 0)
+				else if(st == 304 && acfg::vrangeops == 0)
 				{
 					m_pStorage->SetupComplete();
 					m_DlState = STATE_FINISHJOB;
@@ -365,6 +426,8 @@ struct tDlJob
 					m_nRest = atoofft(h.h[header::CONTENT_LENGTH]);
 					m_DlState = STATE_PROCESS_DATA;
 				}
+
+				// ok, can pass the data to the file handler
 				h.set(header::XORIG, RemoteUri());
 				fileitem_base::SuccessWithTransErrorFlag headStored = m_pStorage->DownloadStartedStoreHeader(h, inBuf.rptr(), bHotItem );
 				if(!headStored && headStored.xdata)
@@ -471,6 +534,7 @@ inline void dlcon::BlacklistMirror(tDlJobPtr & job, cmstring &msg)
 inline bool dlcon::SetupJobConfig(tDlJobPtr &job, mstring *pReasonMsg)
 {
 	LOGSTART("dlcon::SetupJobConfig");
+
 	// using backends? Find one which is not blacklisted
 
 	MYMAP<MYSTD::pair<cmstring,cmstring>, mstring>::const_iterator bliter;
@@ -532,12 +596,14 @@ inline void dlcon::EnqJob(tDlJob *todo)
 void dlcon::AddJob(tFileItemPtr m_pItem, 
 		const acfg::tRepoData *pBackends, const MYSTD::string & sPatSuffix)
 {
-	EnqJob(new tDlJob(this, m_pItem, pBackends, sPatSuffix));
+	EnqJob(new tDlJob(this, m_pItem, pBackends, sPatSuffix,
+			m_bManualMode ? ACFG_REDIRMAX_DEFAULT : acfg::redirmax));
 }
 
 void dlcon::AddJob(tFileItemPtr m_pItem, tHttpUrl hi)
 {
-	EnqJob(new tDlJob(this, m_pItem, hi));
+	EnqJob(new tDlJob(this, m_pItem, hi,
+			m_bManualMode ? ACFG_REDIRMAX_DEFAULT : acfg::redirmax));
 }
 
 void dlcon::SignalStop()
@@ -688,6 +754,7 @@ inline UINT dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueu
 			}
 			else if (r == 0)
 			{
+				dbgline;
 				sErrorMsg = "502 Connection closed";
 				return EFLAG_LOST_CON;
 			}
@@ -716,7 +783,7 @@ inline UINT dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueu
 			{
 
 				ldbg("Processing job for " << qActive.front()->RemoteUri());
-				UINT res = qActive.front()->ProcessIncomming(m_inBuf);
+				UINT res = qActive.front()->ProcessIncomming(m_inBuf, false);
 				ldbg(
 						"... incoming data processing result: " << res
 						<< ", emsg: " << qActive.front()->sErrorMsg);
@@ -726,8 +793,9 @@ inline UINT dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueu
 
 				if (HINT_DONE & res)
 				{
+
 					// just in case that server damaged the last response body
-					con->NameLastFile(qActive.front()->m_pStorage->m_sPathRel);
+					con->KnowLastFile(WEAK_PTR<fileitem_base>(qActive.front()->m_pStorage));
 
 					qActive.pop_front();
 					if (HINT_DISCON & res)
@@ -746,15 +814,30 @@ inline UINT dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueu
 					LOG("Extract more responses");
 					continue;
 				}
-				else // error handling, pass to main loop
+
+				if (HINT_TGTCHANGE & res)
 				{
-
-					if(HINT_KILL_LAST_FILE & res)
-						con->KillLastFile();
-
-					setIfNotEmpty(sErrorMsg, qActive.front()->sErrorMsg);
-					return res;
+					/* target was rewritten by the job handler, but there might be more redirections
+				    in the pipe which we can analyze and use ASAP to avoid later requests.
+				    Process as much as we can (carefully). */
+					tDljQueue::iterator it = qActive.begin();
+					for(++it; it != qActive.end(); ++it)
+					{
+						UINT rr = (**it).ProcessIncomming(m_inBuf, true);
+						if( HINT_TGTCHANGE != rr )
+						{
+							m_nDisablePling=30;
+							return (HINT_TGTCHANGE|HINT_DISCON);
+						}
+					}
+					return HINT_TGTCHANGE;
 				}
+
+				// else case: error handling, pass to main loop
+				if(HINT_KILL_LAST_FILE & res)
+					con->KillLastFile();
+				setIfNotEmpty(sErrorMsg, qActive.front()->sErrorMsg);
+				return res;
 			}
 			return HINT_DONE; // input buffer consumed
 		}
@@ -905,7 +988,7 @@ void dlcon::WorkLoop()
         				&& ( cjob->GetPeerName() != con->GetHostname()
         				|| cjob->GetPeerPort() != con->GetPort()) )
         		{
-        			LOG("host mismatch, stop requesting");
+        			LOG("host mismatch, stop sending requesting for now");
         			bStopRequesting=true;
         			break;
         		}
@@ -915,6 +998,12 @@ void dlcon::WorkLoop()
         			LOG("request added to buffer");
         			inpipe.push_back(cjob);
         			m_qNewjobs.pop_front();
+
+        			if(m_nDisablePling >0)
+        			{
+        				bStopRequesting=true;
+        				--m_nDisablePling;
+        			}
         		}
         	}
         }
@@ -933,19 +1022,22 @@ void dlcon::WorkLoop()
         loopRes=ExchangeData(sErrorMsg, con, inpipe);
         ldbg("loopRes: "<< loopRes);
 
-        // no matter what happened, that stop flag is now irrelevant
-        if(inpipe.empty())
-        {
-        	dbgline;
-        	bStopRequesting=false;
-        }
+        if (inpipe.empty())
+		{
+        	// all requests have been processed (client done, or pipeline stall, who cares)
+			dbgline;
 
-        if(0==(loopRes&(HINT_DISCON|EFLAG_JOB_BROKEN|EFLAG_LOST_CON|EFLAG_MIRROR_BROKEN))
-        		&& con && inpipe.empty()) // no error bits set, not busy -> recycle it when idle
-        {
-        	dbgline;
-			tcpconnect::RecycleIdleConnection(con);
-			continue;
+			// no matter what happened, that stop flag is now irrelevant
+			bStopRequesting = false;
+
+			// no error bits set, not busy -> this connection is still good, recycle properly
+			UINT all_err = HINT_DISCON | EFLAG_JOB_BROKEN | EFLAG_LOST_CON | EFLAG_MIRROR_BROKEN;
+			if (con && !(loopRes & all_err))
+			{
+				dbgline;
+				tcpconnect::RecycleIdleConnection(con);
+				continue;
+			}
 		}
 
         if( (HINT_DISCON|EFLAG_LOST_CON) & loopRes)
@@ -954,6 +1046,14 @@ void dlcon::WorkLoop()
         	con.reset();
         }
 
+        if ( loopRes & HINT_TGTCHANGE )
+        {
+        	// short queue has been rewritten, needs to play back everything
+        	// if not reset above -> should be in good shape
+        	tcpconnect::RecycleIdleConnection(con);
+        	goto move_jobs_back_to_q;
+        }
+
 		if ((EFLAG_LOST_CON & loopRes) && !inpipe.empty())
 		{
 			// disconnected by OS... give it a chance, or maybe not...
@@ -1008,10 +1108,10 @@ void dlcon::WorkLoop()
         		setLockGuard;
         		CleanRunning(m_qNewjobs);
         	}
-
         }
 
-        // if the job was not dropped, push it back into the task queue
+        move_jobs_back_to_q:
+        // for the jobs that were not finished and/or dropped, move them back into the task queue
         {
         	setLockGuard;
         	m_qNewjobs.insert(m_qNewjobs.begin(), inpipe.begin(), inpipe.end());
