@@ -185,7 +185,7 @@ void expiration::HandlePkgEntry(const tRemoteFileInfo &entry, bool bUnpackForCsu
 					if(GetFlags(sPathRel).parseignore)
 					{
 						SendFmt() << sPathRel << ": incorrect or obsolete contents but marked as "
-								"alternative version of another index file (ignoring)";
+								"alternative version of another index file <font color=orange>(ignoring)</font>";
 					}
 					else
 					{
@@ -560,7 +560,7 @@ void expiration::Action(const string & cmd)
 
 void expiration::PurgeMaintLogs()
 {
-	tStrDeq logs = ExpandFilePattern(acfg::logdir + SZPATHSEP"maint_*.log");
+	tStrDeq logs = ExpandFilePattern(acfg::logdir + SZPATHSEP"maint_*.log*");
 	if (logs.size() > 2)
 		SendChunk(
 				"Found required cleanup tasks: purging maintanence logs...<br>\n");
