@@ -16,6 +16,7 @@
 using namespace MYSTD;
 
 #define MAXTEMPDELAY acfg::maxtempdelay // 27
+mstring sReplDir("_altStore" SZPATHSEP);
 
 header const & fileitem_base::GetHeaderUnlocked()
 {
@@ -30,7 +31,7 @@ string fileitem_base::GetHttpMsg()
 	return m_head.frontLine;
 }
 
-time_t fileitem_base::m_nEarliestExpiration(cleaner::never());
+time_t fileitem_base::m_nEarliestExpiration(END_OF_TIME);
 
 fileitem_base::fileitem_base(MYSTD::string sPath) :
 	condition(),
@@ -44,7 +45,8 @@ fileitem_base::fileitem_base(MYSTD::string sPath) :
 	m_filefd(-1),
 	m_nDlRefsCount(0),
 	m_status(FIST_FRESH),
-	m_nTimeExpireAt(0)
+	m_nTimeExpireAt(0),
+	m_nTimeDlDone(END_OF_TIME)
 {
 
 	// FIXME: this sucks, belongs into the fileitem
@@ -113,18 +115,18 @@ fileitem_base::fileitem_base() :
 	m_filefd(-1),
 	m_nDlRefsCount(0),
 	m_status(FIST_FRESH),
-	m_nTimeExpireAt(0)
+	m_nTimeExpireAt(0),
+	m_nTimeDlDone(END_OF_TIME)
 {
 }
 
 fileitem_base::~fileitem_base()
 {
-	setLockGuard;
+	//setLockGuard;
 //	m_head.clear();
 	checkforceclose(m_filefd);
 }
 
-
 void fileitem_base::IncDlRefCount()
 {
 	setLockGuard;
@@ -344,7 +346,7 @@ inline void _LogWithErrno(const char *msg, const string & sFile)
 #define SET_FRONTLINE(ret, x) ret="HTTP/1.1 "; ret+=x;
 #define SETERROR(x) { m_bAllowStoreData=false; SET_FRONTLINE(m_head.frontLine, x); \
 	m_head.set(header::XORIG, h.h[header::XORIG]); \
-    m_status=FIST_LOCALERROR; _LogWithErrno(x, m_sPathRel); }
+    m_status=FIST_LOCALERROR; m_nTimeDlDone=GetTime(); _LogWithErrno(x, m_sPathRel); }
 #define SETERRORKILLFILE(x) { SETERROR(x); goto kill_file; }
 #define BOUNCE(x) { SETERROR(x); return false; }
 
@@ -506,24 +508,18 @@ fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const char *
 		{
 		// -> kill cached file ASAP
 			m_bAllowStoreData=false;
-			m_head.set(header::XORIG, h.h[header::XORIG]);
+			m_head.copy(h, header::XORIG);
 			SETERRORKILLFILE("503 Server disagrees on file size, cleaning up");
 		}
 		break;
-// use the Location header and be sure the server does not mess around with Location and other codes
-	case 301:
-	case 302:
-	case 303:
-	case 307:
-		m_head.set(header::LOCATION, h.h[header::LOCATION]);
-		// fall-through
 	default:
 		m_head.type=header::ANSWER;
-		m_head.set(header::XORIG, h.h[header::XORIG]);
+		m_head.copy(h, header::XORIG);
+		m_head.copy(h, header::LOCATION);
 		if(bForcedRestart)
 		{
-			// got a redirection from another mirror? or an error? cannot handle it yet.
-			// XXX: concept change required
+			// got an error from the replacement mirror? cannot handle it properly
+			// because some job might already have started returning the data
 			USRDBG(0, "Cannot restart, HTTP code: " << serverStatus);
 			BOUNCE(h.getCodeMessage());
 		}
@@ -648,6 +644,7 @@ fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const char *
 	unlink(sHeadPath.c_str());
 	
 	m_status=FIST_LOCALERROR;
+	m_nTimeDlDone=GetTime();
 	return false;
 }
 
@@ -675,6 +672,7 @@ bool fileitem_with_storage::StoreFileData(const char *data, unsigned int size)
 		else
 		{
 			m_status = FIST_COMPLETE;
+			m_nTimeDlDone=GetTime();
 
 			if (acfg::debug & LOG_MORE)
 				aclog::misc(tSS() << "Download of " << m_sPathRel << " finished");
@@ -703,6 +701,7 @@ bool fileitem_with_storage::StoreFileData(const char *data, unsigned int size)
 					aclog::errnoFmter efmt("HTTP/1.1 503 ");
 					m_head.frontLine = efmt;
 					m_status=FIST_LOCALERROR;
+					m_nTimeDlDone=GetTime();
 					_LogWithErrno(efmt.c_str(), m_sPathRel);
 					return false;
 				}
@@ -720,159 +719,165 @@ bool fileitem_with_storage::StoreFileData(const char *data, unsigned int size)
 	return true;
 }
 
-
-struct tFileRefEntry
-{
-	int nRefCount;
-	tFileItemPtr ref;
-	bool m_bNeedsDelayedExpiration;
-	tFileRefEntry() : nRefCount(0), m_bNeedsDelayedExpiration(false) { };
-};
-static MYMAP<string, tFileRefEntry> mapItems;
+//typedef MYSTD::pair<int,tFileItemPtr> tFileRefEntry;
+//typedef MYSTD::multimap<string, tFileRefEntry> tFiGlobMap;
+static tFiGlobMap mapItems;
 lockable mapLck;
 
+fileitem::fileitem(cmstring &s) : fileitem_with_storage(s), m_globRef(mapItems.end())
+{};
+
 void fileitem::Unreg()
 {
 	LOGSTART("fileitem::Unreg");
 
-   lockguard managementLock(mapLck); 
-	
-   MYMAP<string,tFileRefEntry>::iterator entry = mapItems.find(m_sPathRel);
-   if(mapItems.end() == entry)
-   {
-	   aclog::err("INTERNAL ERROR, attempt to unregister non-existing download item");
-	   return;
-   }
-   if(entry->second.ref.get() != this)
-   {
-	   aclog::err("INTERNAL ERROR, doppelganger running amok");
-	   return;
-   }
-   entry->second.nRefCount--;
-   if(entry->second.nRefCount<=0)
-   {
-	   LOG("*this is last entry, deleting dl/fi mapping");
-	  setLockGuard;
-	  m_status=FIST_ERRNOUSER;
-	  notifyAll();
-	  mapItems.erase(entry);
-   }
+	lockguard managementLock(mapLck);
+	setLockGuard;
+
+	if(m_globRef == mapItems.end())
+	{
+		// not globally registered, just ignore
+		return;
+	}
+
+	m_globRef->second.first--;
+	if (m_globRef->second.first <= 0)
+	{
+		// use the delay timer only if late expiration is needed and it isn't expired already
+		if (m_nTimeExpireAt && GetTime() < m_nTimeExpireAt)
+		{
+			m_nEarliestExpiration = MYSTD::min(m_nTimeExpireAt, m_nEarliestExpiration);
+			g_victor.RunLatestAt(m_nEarliestExpiration);
+			return;
+		}
+
+		LOG("*this is last entry, deleting dl/fi mapping");
+		m_status = FIST_ERRNOUSER;
+		notifyAll();
+		mapItems.erase(m_globRef);
+		m_globRef = mapItems.end();
+	}
 }
 
-tFileItemPtr fileitem::GetRegisteredFileItem(MYSTD::string sPathKey)
+
+tFileItemPtr fileitem::GetRegisteredFileItem(cmstring &sPathKey, bool bConsiderAltStore)
 {
 	LOGSTART2s("fileitem::GetFileItem", sPathKey);
-	tFileItemPtr p;
+
 	MYTRY
 	{
-		p.reset(new fileitem(sPathKey)); // let it create real target path
+		fileitem *p = new fileitem(sPathKey); // need one because it will convert the path string to the proper value needed as key
+		tFileItemPtr sp(p);
 		lockguard lockGlobalMap(mapLck);
 
-		map<string, tFileRefEntry>::iterator it=mapItems.find(p->m_sPathRel);
+		tFiGlobMap::iterator it=mapItems.find(p->m_sPathRel);
 		if(it!=mapItems.end())
 		{
-			it->second.nRefCount++;
-			LOG("Sharing existing file item");
-			return it->second.ref;
-		}
-		// ok, keep the new item
+			if (bConsiderAltStore)
+			{
+				// detect items that got stuck somehow
+				time_t now(GetTime());
+				time_t extime(now - acfg::stucksecs);
+				if (it->second.second.get()->m_nTimeDlDone < extime)
+				{
+					// try to find its sibling which is in good state?
+					for (; it!=mapItems.end() && it->first == p->m_sPathRel; ++it)
+					{
+						if (it->second.second.get()->m_nTimeDlDone >= extime)
+						{
+							it->second.first++;
+							LOG("Sharing an existing REPLACEMENT file item");
+							return it->second.second;
+						}
+					}
+					// ok, then create a modded name version in the replacement directory
+					mstring sKeyPathOrig(p->m_sPathRel);
+					replaceChars(p->m_sPathRel, "/\\", '_');
+					p->m_sPathRel.insert(0, sReplDir + ltos(getpid()) + "_" + ltos(now))+"_";
+					p->m_sPathAbs = acfg::cacheDirSlash+p->m_sPathRel;
 
-		LOG("Created new file item");
-		tFileRefEntry & entry = mapItems[p->m_sPathRel];
-		LOG("Registering new file item...");
-		if(!entry.ref)
-		{
-			entry.ref=p;
-			entry.nRefCount++;
-			LOG("New reference count: " << entry.nRefCount);
+					LOG("Registering a new REPLACEMENT file item...");
+					p->m_globRef = mapItems.insert(make_pair(sKeyPathOrig, tFileRefEntry(1,sp)));
+					return sp;
+				}
+			}
+			LOG("Sharing existing file item");
+			it->second.first++;
+			return it->second.second;
 		}
-		return entry.ref;
+		LOG("Registering the NEW file item...");
+		p->m_globRef = mapItems.insert(make_pair(p->m_sPathRel, tFileRefEntry(1,sp)));
+		return sp;
 	}
 	MYCATCH(MYSTD::bad_alloc&)
 	{
-		p.reset();
 	}
-	return p;
+	return tFileItemPtr();
 }
 
+// make the fileitem globally accessible
 bool fileitem::RegisterFileItem(SHARED_PTR<fileitem> spCustomFileItem)
-		{
-			if (!spCustomFileItem || spCustomFileItem->m_sPathRel.empty())
-				return false;
-
-			lockguard lockGlobalMap(mapLck);
-
-			map<string, tFileRefEntry>::iterator it = mapItems.find(spCustomFileItem->m_sPathRel);
-			if (it != mapItems.end())
-				return false; // conflict, another agent is already active
-
-			tFileRefEntry & entry = mapItems[spCustomFileItem->m_sPathRel];
-			entry.ref = spCustomFileItem;
-			entry.nRefCount = 1;
-
-			return true;
-		}
-
-// Opposite of GetFileItem, unregisters existing file item now or as soon as possible.
-void fileitem::UnregOneASAP(tFileItemPtr pItem)
 {
-	if (!pItem)
-		return;
-	// download never started or no delayed expiration needed -> release now. Or expired long ago.
-	if (!pItem->m_nTimeExpireAt || GetTime() >= pItem->m_nTimeExpireAt)
-	{
-		pItem->Unreg();
-		return;
-	}
+	if (!spCustomFileItem || spCustomFileItem->m_sPathRel.empty())
+		return false;
+
 	lockguard lockGlobalMap(mapLck);
-	MYMAP<string, tFileRefEntry>::iterator entry = mapItems.find(pItem->m_sPathRel);
-	// not existing? -> not for us. Still enough references? -> let this one go
-	if (mapItems.end() == entry || entry->second.nRefCount>1)
-	{
-		lockGlobalMap.unLock();
-		pItem->Unreg();
-		return;
-	}
-	// now the item's refCount is by one higher then it should be, set the flag instead of --
-	entry->second.m_bNeedsDelayedExpiration=true;
 
-	m_nEarliestExpiration = MYSTD::min(pItem->m_nTimeExpireAt, m_nEarliestExpiration);
-	g_victor.RunLatestAt(m_nEarliestExpiration);
+	if(ContHas(mapItems, spCustomFileItem->m_sPathRel))
+		return false; // conflict, another agent is already active
 
+	spCustomFileItem->m_globRef = mapItems.insert(make_pair(spCustomFileItem->m_sPathRel,
+			tFileRefEntry(1, spCustomFileItem)));
+
+	return true;
 }
 
+
 // this method is supposed to be awaken periodically and detect items with ref count manipulated by
 // the request storm prevention mechanism. Items shall be be dropped after some time if no other
 // thread but us is using them.
 time_t fileitem::DoDelayedUnregAndCheck()
 {
 	lockguard lockGlobalMap(mapLck);
-	MYMAP<string, tFileRefEntry>::iterator it, here;
+	tFiGlobMap::iterator it, here;
 
 	time_t now=GetTime();
 
 	if(now<m_nEarliestExpiration) // unreachable anyhow
 		return m_nEarliestExpiration;
 
-	m_nEarliestExpiration = g_victor.never(); // the correct time will be found below
+	m_nEarliestExpiration = END_OF_TIME; // the correct time will be found below
 
 	for(it=mapItems.begin(); it!=mapItems.end();)
 	{
 		here=it++;
-		if(!here->second.m_bNeedsDelayedExpiration)
+
+		// not needed or not ready yet
+		if (!here->second.second->m_nTimeExpireAt)
 			continue;
-		// still active or young item? skip, but find the earliest time to expire
-		if(now < here->second.ref->m_nTimeExpireAt || here->second.nRefCount>1)
+
+		m_nEarliestExpiration=min(here->second.second->m_nTimeExpireAt, m_nEarliestExpiration);
+
+		if (now < here->second.second->m_nTimeExpireAt)
+			continue;
+
+		// became busy again? Ignore this entry, it's new master will take care of the deletion ASAP
+		if(here->second.first>0)
+			continue;
+
+		// ok, unused and delay is over. Destroy with the same sequence as Unreg does, care about
+		// live time.
+		tFileItemPtr local_ptr(here->second.second);
+		fileitem *p = static_cast<fileitem*>(local_ptr.get());
 		{
-			m_nEarliestExpiration=min(here->second.ref->m_nTimeExpireAt, m_nEarliestExpiration);
-			continue;
+			lockguard g(p);
+			p->m_status = FIST_ERRNOUSER;
+			p->m_globRef = mapItems.end();
+			mapItems.erase(here);
 		}
-		// ok, unused and delay is over. Destroy with the same sequence as Unreg... does.
-		lockguard g(here->second.ref.get());
-		here->second.ref->m_status=FIST_ERRNOUSER;
-		here->second.ref->notifyAll();
-		g.unLock();
-		mapItems.erase(here);
+		// p still alive, through local_ptr or external references to it
+		p->notifyAll();
+
 	}
 	return m_nEarliestExpiration;
 }
@@ -909,6 +914,49 @@ ssize_t fileitem_with_storage::SendData(int out_fd, int in_fd, off_t &nSendPos,
 #endif
 }
 
+void dump_handler(int) {
+	fileitem::dump_status();
+}
+
+void fileitem::dump_status()
+{
+	tSS fmt;
+	aclog::err("File descriptor table:\n", NULL);
+	for(tFiGlobMap::iterator it=mapItems.begin(); it!=mapItems.end(); ++it)
+	{
+		fmt.clear();
+		fmt << "FREF: " << it->first << " [" << it->second.first << "]:\n";
+		if(! it->second.second)
+		{
+			fmt << "\tBAD REF!\n";
+			continue;
+		}
+		else
+		{
+			fmt << "\t" << it->second.second->m_sPathRel
+					<< "\n\t" << it->second.second->m_sPathAbs
+					<< "\n\tPos.dl.agents: " << it->second.second->m_nDlRefsCount
+					<< "\n\tState: " << it->second.second->m_status
+					<< "\n\tPos: " << it->second.second->m_nIncommingCount << " , "
+					<< it->second.second->m_nRangeLimit << " , "
+					<< it->second.second->m_nSizeChecked << " , "
+					<< it->second.second->m_nSizeSeen
+			<< "\n\tExp: " << it->second.second->m_nEarliestExpiration << " , "
+			<< it->second.second->m_nTimeExpireAt << "\n\n";
+		}
+		aclog::err(fmt.c_str(), NULL);
+	}
+}
+
+fileitem_with_storage::~fileitem_with_storage()
+{
+	if(startsWith(m_sPathRel, sReplDir))
+	{
+		unlink(m_sPathAbs.c_str());
+		unlink((m_sPathAbs+".head").c_str());
+	}
+}
+
 #endif // MINIBUILD
 
 FiStatus fileitem_base::WaitForFinish(int *httpCode)
@@ -920,3 +968,4 @@ FiStatus fileitem_base::WaitForFinish(int *httpCode)
 		*httpCode=m_head.getStatus();
 	return m_status;
 }
+
