@@ -186,6 +186,7 @@ public:
 	// never used to store data
 	SuccessWithTransErrorFlag DownloadStartedStoreHeader(const header &, const char *, bool) {return false;};
 	bool StoreFileData(const char *, unsigned int) {return false;};
+	header & HeadRef() { return m_head; }
 };
 
 ssize_t sendfile_generic(int out_fd, int in_fd, off_t *offset, size_t count)
@@ -249,8 +250,8 @@ job::job(header *h, con *pParent) :
 	m_nAllDataCount(0),
 	m_nChunkRemainingBytes(0),
 	m_type(rechecks::FILE_INVALID),
-	m_nReqRangeFrom(-1), m_nReqRangeTo(-1),
-	m_nRedirRemaining(acfg::redirmax)
+	m_nReqRangeFrom(-1), m_nReqRangeTo(-1)
+//,m_nRedirRemaining(acfg::redirmax)
 {
 	LOGSTART2("job::job", "job creating, " << m_pReqHead->frontLine << " and this: " << uintptr_t(this));
 }
@@ -269,9 +270,8 @@ job::~job()
 			m_nAllDataCount, bErr);
 	
 	checkforceclose(m_filefd);
-
-	fileitem::UnregOneASAP(m_pItem);
-
+	if(m_pItem)
+		m_pItem->Unreg();
 	delete m_pReqHead;
 }
 
@@ -493,43 +493,6 @@ inline bool job::ParseRange()
 	return false;
 }
 
-bool job::CreateRedirect(header & respHead)
-{
-	int st = respHead.getStatus();
-	if (m_nRedirRemaining <= 0 || (st != 301 && st != 302 && st != 307) // redir limit reached or not redirect
-	|| !respHead.h[header::LOCATION] || !respHead.h[header::LOCATION][0]) // or nonsense
-		return false;
-
-	MYTRY
-	{
-		header *xheader = new header(*m_pReqHead);
-		// it's previous server's job to not send any crap. Just make sure that the format is right...
-		xheader->frontLine = string("GET ");
-		for (const char *p = respHead.h[header::LOCATION]; *p; ++p)
-		{
-			if (isspace((unsigned char) *p))
-			{
-				xheader->frontLine += "%";
-				xheader->frontLine += BytesToHexString((uint8_t*) p, 1);
-			}
-			else
-				xheader->frontLine += *p;
-		}
-		xheader->frontLine += " HTTP/1.1";
-		job *pReplacement = m_pParentCon->AddNewJob(xheader);
-		if (!pReplacement)
-			return false;
-		pReplacement->m_nRedirRemaining = m_nRedirRemaining - 1;
-	}
-	MYCATCH(...)
-	{
-		return false;
-	}
-	return true;
-}
-
-
-
 void job::PrepareDownload() {
 
     LOGSTART("job::PrepareDownload");
@@ -667,7 +630,7 @@ void job::PrepareDownload() {
 
 		bForceFreshnessChecks = ( ! acfg::offlinemode && m_type == FILE_INDEX);
 
-    	m_pItem=fileitem::GetRegisteredFileItem(m_sFileLoc);
+    	m_pItem=fileitem::GetRegisteredFileItem(m_sFileLoc, bForceFreshnessChecks);
 
 	}
 	MYCATCH(MYSTD::out_of_range&) // better safe...
@@ -823,7 +786,7 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 					m_sOrigUrl=h.h[header::XORIG];
 				// must be something like "HTTP/1.1 403 Forbidden"
 				if(h.frontLine.length()>9 && h.getStatus()!=200)
-					SetErrorResponse(h.frontLine.c_str()+9);
+					SetErrorResponse(h.frontLine.c_str()+9, h.h[header::LOCATION]);
 				else // good ungood? confused somewhere?!
 					SetErrorResponse("500 Unknown error");
 				return R_AGAIN;
@@ -874,9 +837,6 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 					m_state=STATE_SEND_BUFFER;
 					m_backstate=STATE_HEADER_SENT; // could be changed while creating header
 
-					if(CreateRedirect(respHead))
-						return R_DONE;
-
 					const char *szErr = BuildAndEnqueHeader(fistate, nGoodDataSize, respHead);
 					if(szErr) THROW_ERROR(szErr);
 					USRDBG(REPLEVEL, "Prepared response header for user: \n" << m_sendbuf );
@@ -1076,7 +1036,6 @@ inline const char * job::BuildAndEnqueHeader(const FiStatus &fistate,
 			bool bFreshnessForced = (m_type != rechecks::FILE_INDEX
 				|| m_pReqHead->h[header::ACNGFSMARK] || !pIfmo);
 
-			// this plays well with lighttpd <-> Apt, but may not work with the alternative time format
 			struct tm tm1, tm2;
 			bool bIfModSeenAndChecked=false;
 			if(pIfmo && header::ParseDate(pIfmo, &tm1) && header::ParseDate(pLastMo, &tm2))
@@ -1192,10 +1151,12 @@ FiStatus job::_SwitchToPtItem(const MYSTD::string &fileLoc)
 }
 
 
-void job::SetErrorResponse(const char * errorLine)
+void job::SetErrorResponse(const char * errorLine, const char *szLocation)
 {
 	LOGSTART2("job::SetErrorResponse", errorLine);
 
+	if(m_pItem)
+		m_pItem->Unreg();
 
 	class erroritem: public tGeneratedFitemBase
 	{
@@ -1213,12 +1174,10 @@ void job::SetErrorResponse(const char * errorLine)
 		}
 	};
 
-	tFileItemPtr pNew(static_cast<fileitem_base*>(new erroritem("noid", errorLine)));
-
-	fileitem::UnregOneASAP(m_pItem);
-
+	erroritem *p = new erroritem("noid", errorLine);
+	p->HeadRef().set(header::LOCATION, szLocation);
+	tFileItemPtr pNew(static_cast<fileitem_base*>(p));
 	m_pItem=pNew;
-
 	//aclog::err(tSS() << "fileitem is now " << uintptr_t(m_pItem.get()));
 	m_state=STATE_SEND_MAIN_HEAD;
 }
