@@ -194,6 +194,8 @@ void DispatchAndRunMaintTask(cmstring &cmd, int conFD, const char * szAuthLine)
 				pWorker = new expiration(conFD, expiration::listDamaged);
 			else if(StrHas(cmd, "justRemoveDamaged="))
 				pWorker = new expiration(conFD, expiration::purgeDamaged);
+			else if(StrHas(cmd, "justTruncDamaged="))
+				pWorker = new expiration(conFD, expiration::truncDamaged);
 			else if(cmd.find("doImport=")!=stmiss)
 				pWorker=new pkgimport(conFD);
 			else if(cmd.find("doMirror=")!=stmiss)
@@ -211,19 +213,20 @@ void DispatchAndRunMaintTask(cmstring &cmd, int conFD, const char * szAuthLine)
 					pWorker=new tDeleter(conFD);
 			}
 			else if(cmd.find("doCount=")!=stmiss)
-				pWorker = new tStaticFileSend(conFD, "report.html", "text/html", "200");
+				pWorker = new tStaticFileSend(conFD, "report.html", "text/html", "200 OK");
 			else // ok... just show the default page
 				goto l_show_cnc;
 		}
 		else if (cmd==acfg::reportpage)
 		{
 			l_show_cnc:
-			pWorker = new tStaticFileSend(conFD, "report.html", "text/html", "200");
+			pWorker = new tStaticFileSend(conFD, "report.html", "text/html", "200 OK");
 		}
 		else if (cmd == "/style.css")
-			pWorker = new tStaticFileSend(conFD, "style.css", "text/css", "200");
+			pWorker = new tStaticFileSend(conFD, "style.css", "text/css", "200 OK");
 		else
-			pWorker = new tStaticFileSend(conFD, "userinfo.html", "text/html", "404");
+			pWorker = new tStaticFileSend(conFD, "userinfo.html", "text/html",
+					"404 Usage Information");
 
 		if(pWorker)
 			pWorker->Run(cmd);
