@@ -426,4 +426,55 @@ int GetSimilarity(cmstring& wanted, cmstring& candidate)
 }
 */
 
+
+
 #endif
+
+
+#define _inv MAX_VAL(uint_fast16_t)
+
+uint_fast16_t hexmap[] = {
+                _inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,
+                _inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,
+                _inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,
+                0,1,2,3,4,5,6,7,8,9,_inv,_inv,_inv,_inv,_inv,_inv,
+                _inv,10,11,12,13,14,15,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,
+                _inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,
+                _inv,10,11,12,13,14,15,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,
+                _inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,
+                _inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,
+                _inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,
+                _inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,
+                _inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,
+                _inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,
+                _inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,
+                _inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,
+                _inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv
+                };
+
+bool CsAsciiToBin(const char *a, uint8_t b[], unsigned short binLength)
+{
+	const unsigned char *uA = (const unsigned char*) a;
+	for(int i=0; i<binLength;i++)
+	{
+		if(hexmap[uA[i*2]]>15 || hexmap[uA[i*2+1]] > 15)
+			return false;
+		b[i]=hexmap[uA[i*2]] * 16 + hexmap[uA[i*2+1]];
+	}
+	return true;
+}
+
+string BytesToHexString(const uint8_t sum[], unsigned short lengthBin)
+{
+	string sRet;
+	char hexmap[] =
+	{ '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd',
+			'e', 'f' };
+	for (int i=0; i<lengthBin; i++)
+	{
+		sRet+=(char) hexmap[sum[i]>>4];
+		sRet+=(char) hexmap[sum[i]&0xf];
+	}
+
+	return sRet;
+}
