@@ -22,7 +22,7 @@ using namespace MYSTD;
 
 pkgmirror::pkgmirror(int fd): tCacheMan(fd),
 m_bCalcSize(false), m_bSkipIxUpdate(false), m_bDoDownload(false), m_bAsNeeded(false),
-m_totalSize(0), m_totalHave(0), m_pDeltaSrc(NULL)
+m_bUseDelta(false), m_totalSize(0), m_totalHave(0), m_pDeltaSrc(NULL), m_repCutLen(0)
 {
 	m_sTypeName="Mirroring";
 }
@@ -259,6 +259,10 @@ void pkgmirror::Action(const string &cmd)
 
 		for (tStrSet::iterator it = srcs.begin(); it != srcs.end(); it++)
 		{
+#ifdef DEBUG
+			if(LOG_MORE&acfg::debug)
+				SendFmt() << "mirror check: " << *it;
+#endif
 			off_t needBefore=(m_totalSize-m_totalHave);
 
 			ParseAndProcessIndexFile(*this, *it, GuessIndexTypeFromURL(*it));
@@ -315,6 +319,15 @@ inline bool pkgmirror::ConfigDelta(cmstring &sPathRel)
 	{
 		vname.resize(m_repCutLen);
 		const acfg::tRepoData *pRepo = acfg::GetBackendVec(vname);
+#ifdef DEBUG
+		if(acfg::debug & LOG_MORE)
+		{
+			if(!pRepo)
+				SendFmt() << "hm, no delta provider for " << sPathRel;
+		}
+		else
+			SendFmt()<< "hm, delta for " << sPathRel << " is what? " <<pRepo->m_deltasrc.sHost;
+#endif
 		if (pRepo && !pRepo->m_deltasrc.sHost.empty())
 			m_pDeltaSrc = &pRepo->m_deltasrc;
 	}
