@@ -71,8 +71,8 @@ void tStaticFileSend::Run(const string &cmd)
 				"please contact the system administrator.";
 	}
 	tSS buf(1023);
-	buf << "HTTP/1.1 " << (m_sHttpCode ? m_sHttpCode : "200")
-			<< " OK\r\nConnection: close\r\nContent-Type: "
+	buf << "HTTP/1.1 " << (m_sHttpCode ? m_sHttpCode : "200 OK")
+			<< "\r\nConnection: close\r\nContent-Type: "
 			<< (m_sMimeType?m_sMimeType:"text/html")
 			<< "\r\nContent-Length: " << contents.length() << "\r\n\r\n";
 	SendRawData(buf.rptr(), buf.size(), MSG_MORE);
@@ -113,7 +113,7 @@ void tDeleter::ModContents(mstring & contents, cmstring &cmd)
 	for(tStrVecIterConst it=files.begin(); it!=files.end(); it++)
 	{
 		if(it->find_first_of(BADCHARS)!=stmiss  // what the f..., XSS attempt?
-				|| stmiss != it->find("..") )
+		 || rechecks::Match(*it, rechecks::NASTY_PATH))
 		{
 			contents.clear();
 			return;
@@ -137,11 +137,11 @@ void tDeleter::ModContents(mstring & contents, cmstring &cmd)
 	if(!bConfirmMode)
 	{
 		StrSubst(contents, "$VISACTION", "visible");
-		StrSubst(contents, "$VISQUESTION", "hidden");
+		StrSubst(contents, "$VISQUESTION", "hidden;height:0px;");
 	}
 	else // just confirm
 	{
-		StrSubst(contents, "$VISACTION", "hidden");
+		StrSubst(contents, "$VISACTION", "hidden;height:0px;");
 		StrSubst(contents, "$VISQUESTION", "visible");
 	}
 }
