@@ -167,7 +167,7 @@ void tcpconnect::Disconnect()
 		m_pConnStateObserver->JobRelease();
 
 	m_pConnStateObserver=NULL;
-	m_lastFileKeyPath.clear();
+	m_lastFile.reset();
 
 	forceShutdownClose(m_conFd);
 }
@@ -326,21 +326,16 @@ time_t tcpconnect::ExpireCache()
 			++it;
 	}
 
-	return spareConPool.empty() ? g_victor.never() : GetTime()+TIME_SOCKET_EXPIRE_CLOSE/4+1;
+	return spareConPool.empty() ? END_OF_TIME : GetTime()+TIME_SOCKET_EXPIRE_CLOSE/4+1;
 }
 
 void tcpconnect::KillLastFile()
 {
 #ifndef MINIBUILD
-	if (m_lastFileKeyPath.empty())
-		return;
-
-	tFileItemPtr p(fileitem::GetRegisteredFileItem(m_lastFileKeyPath));
+	tFileItemPtr p = m_lastFile.lock();
 	if (!p)
 		return;
-
 	p->SetupClean(true);
-	p->Unreg();
 #endif
 }
 
